# frozen_string_literal: true

module Hermes
  class EntityCode
    include Hermes::Concerns::CommonFieldsConcern

    writer_roles 'entreprise'
    reader_roles 'entreprise'
    owner_field :user_id

    field :code, type: String
    field :expire_at, type: DateTime, default: Time.zone.now + 7.days
    field :entity_id, type: BSON::ObjectId
    field :applicant_id, type: BSON::ObjectId

    validates_presence_of :entity_id
    validates_uniqueness_of :code
    validates_uniqueness_of :applicant_id, scope: [:entity_id], message: 'Un code existe déjà entre le candidat et l\'entreprise', unless: :code_expired?
    validate :expiration_date

    common_indexes *%i[
      code
      expire_at
      entity_id
      applicant_id
    ]

    before_create :set_code, unless: :code?

    def expired?
      expire_at < Time.zone.now.end_of_day
    end

    private

    def set_code
      self.code = SecureRandom.alphanumeric(8).upcase
    end

    def code_expired?
      return true unless self.applicant_id.present?

      existed = Hermes::EntityCode.find_by(applicant_id: self.applicant_id, entity_id: self.entity_id)
      existed && existed.expired?
    end

    def expiration_date
      errors.add :expire_at, 'La date d\'expiration doit être future' unless self.expire_at > Time.zone.now
    end
  end
end
